# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PaymentFlowCreateParams"]


class PaymentFlowCreateParams(TypedDict, total=False):
    amount: Required[int]
    """Required.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000. Can be any integer up to 36 digits.
    """

    counterparty_id: Required[str]
    """Required.

    The ID of a counterparty associated with the payment. As part of the payment
    workflow an external account will be associated with this model.
    """

    currency: Required[str]
    """Required. The currency of the payment."""

    direction: Required[Literal["credit", "debit"]]
    """Required.

    Describes the direction money is flowing in the transaction. Can only be
    `debit`. A `debit` pulls money from someone else's account to your own.
    """

    originating_account_id: Required[str]
    """Required. The ID of one of your organization's internal accounts."""

    due_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """Optional.

    Can only be passed in when `effective_date_selection_enabled` is `true`. When
    set, the due date is shown to your end-user in the pre-built UI as they are
    selecting a payment `effective_date`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import date, datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared.currency import Currency
from .payment_order_type import PaymentOrderType

__all__ = [
    "InvoiceCreateParams",
    "ContactDetails",
    "ContactDetail",
    "CounterpartyBillingAddress",
    "CounterpartyShippingAddress",
    "InvoiceLineItems",
    "InvoiceLineItem",
    "InvoicerAddress",
]


class InvoiceCreateParams(TypedDict, total=False):
    counterparty_id: Required[str]
    """The ID of the counterparty receiving the invoice."""

    due_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """A future date by when the invoice needs to be paid."""

    originating_account_id: Required[str]
    """The ID of the internal account the invoice should be paid to."""

    auto_advance: Optional[bool]
    """
    When true, the invoice will progress to unpaid automatically and cannot be
    edited after entering that state. If the invoice fails to progress to unpaid,
    the errors will be returned and the invoice will not be created.
    """

    contact_details: Iterable[ContactDetail]
    """The invoicer's contact details displayed at the top of the invoice."""

    counterparty_billing_address: Optional[CounterpartyBillingAddress]
    """The counterparty's billing address."""

    counterparty_shipping_address: Optional[CounterpartyShippingAddress]
    """The counterparty's shipping address where physical goods should be delivered."""

    currency: Currency
    """Currency that the invoice is denominated in. Defaults to `USD` if not provided."""

    description: str
    """A free-form description of the invoice."""

    fallback_payment_method: Optional[str]
    """
    When payment_method is automatic, the fallback payment method to use when an
    automatic payment fails. One of `manual` or `ui`.
    """

    ingest_ledger_entries: Optional[bool]
    """Whether to ingest the ledger_entries to populate the invoice line items.

    If this is false, then a line item must be provided. If this is true, line_items
    must be empty. Ignored if ledger_account_settlement_id is empty.
    """

    invoice_line_items: Optional[Iterable[InvoiceLineItem]]
    """An array of invoice line items.

    The API supports a maximum of 50 invoice line items per invoice. If a greater
    number of invoice line items is required, please contact support.
    """

    invoicer_address: Optional[InvoicerAddress]
    """The invoice issuer's business address."""

    ledger_account_settlement_id: Optional[str]
    """The ID of the virtual account the invoice should be paid to."""

    metadata: Optional[Dict[str, str]]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    notification_email_addresses: Optional[List[str]]
    """
    Emails in addition to the counterparty email to send invoice status
    notifications to. At least one email is required if notifications are enabled
    and the counterparty doesn't have an email.
    """

    notifications_enabled: bool
    """
    If true, the invoice will send email notifications to the invoice recipients
    about invoice status changes.
    """

    payment_effective_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """Date transactions are to be posted to the participants' account.

    Defaults to the current business day or the next business day if the current day
    is a bank holiday or weekend. Format: yyyy-mm-dd.
    """

    payment_method: Literal["ui", "manual", "automatic"]
    """The method by which the invoice can be paid.

    `ui` will show the embedded payment collection flow. `automatic` will
    automatically initiate payment based upon the account details of the
    receiving_account id.\nIf the invoice amount is positive, the automatically
    initiated payment order's direction will be debit. If the invoice amount is
    negative, the automatically initiated payment order's direction will be credit.
    One of `manual`, `ui`, or `automatic`.
    """

    payment_type: PaymentOrderType
    """
    One of `ach`, `se_bankgirot`, `eft`, `wire`, `check`, `sen`, `book`, `rtp`,
    `sepa`, `bacs`, `au_becs`, `interac`, `neft`, `nics`,
    `nz_national_clearing_code`, `sic`, `signet`, `provexchange`, `zengin`.
    """

    receiving_account_id: str
    """The receiving account ID. Can be an `external_account`."""

    recipient_email: Optional[str]
    """The email of the recipient of the invoice.

    Leaving this value as null will fallback to using the counterparty's name.
    """

    recipient_name: Optional[str]
    """The name of the recipient of the invoice.

    Leaving this value as null will fallback to using the counterparty's name.
    """

    remind_after_overdue_days: Optional[Iterable[int]]
    """
    Number of days after due date when overdue reminder emails will be sent out to
    invoice recipients.
    """

    virtual_account_id: Optional[str]
    """The ID of the virtual account the invoice should be paid to."""


class ContactDetail(TypedDict, total=False):
    id: Required[str]

    contact_identifier: Required[str]

    contact_identifier_type: Required[Literal["email", "phone_number", "website"]]

    created_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    discarded_at: Required[Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]]

    live_mode: Required[bool]
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: Required[str]

    updated_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]


ContactDetails = ContactDetail
"""This type is deprecated and will be removed in a future release.

Please use ContactDetail instead.
"""


class CounterpartyBillingAddress(TypedDict, total=False):
    country: Required[str]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Required[str]

    locality: Required[str]
    """Locality or City."""

    postal_code: Required[str]
    """The postal code of the address."""

    region: Required[str]
    """Region or State."""

    line2: str


class CounterpartyShippingAddress(TypedDict, total=False):
    country: Required[str]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Required[str]

    locality: Required[str]
    """Locality or City."""

    postal_code: Required[str]
    """The postal code of the address."""

    region: Required[str]
    """Region or State."""

    line2: str


class InvoiceLineItem(TypedDict, total=False):
    name: Required[str]
    """The name of the line item, typically a product or SKU name."""

    unit_amount: Required[int]
    """
    The cost per unit of the product or service that this line item is for,
    specified in the invoice currency's smallest unit.
    """

    description: str
    """An optional free-form description of the line item."""

    direction: str
    """Either `debit` or `credit`.

    `debit` indicates that a client owes the business money and increases the
    invoice's `total_amount` due. `credit` has the opposite intention and effect.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    quantity: int
    """The number of units of a product or service that this line item is for.

    Must be a whole number. Defaults to 1 if not provided.
    """

    unit_amount_decimal: str
    """
    The cost per unit of the product or service that this line item is for,
    specified in the invoice currency's smallest unit. Accepts decimal strings with
    up to 12 decimals
    """


InvoiceLineItems = InvoiceLineItem
"""This type is deprecated and will be removed in a future release.

Please use InvoiceLineItem instead.
"""


class InvoicerAddress(TypedDict, total=False):
    country: Required[str]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Required[str]

    locality: Required[str]
    """Locality or City."""

    postal_code: Required[str]
    """The postal code of the address."""

    region: Required[str]
    """Region or State."""

    line2: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime

import httpx

from .. import _legacy_response
from ..types import ledger_list_params, ledger_create_params, ledger_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.ledger import Ledger

__all__ = ["Ledgers", "AsyncLedgers"]


class Ledgers(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LedgersWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return LedgersWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LedgersWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return LedgersWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Ledger:
        """
        Create a ledger.

        Args:
          name: The name of the ledger.

          description: An optional free-form description for internal use.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/ledgers",
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "metadata": metadata,
                },
                ledger_create_params.LedgerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Ledger,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Ledger:
        """
        Get details on a single ledger.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/ledgers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Ledger,
        )

    def update(
        self,
        id: str,
        *,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Ledger:
        """
        Update the details of a ledger.

        Args:
          description: An optional free-form description for internal use.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          name: The name of the ledger.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/ledgers/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "metadata": metadata,
                    "name": name,
                },
                ledger_update_params.LedgerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Ledger,
        )

    def list(
        self,
        *,
        id: List[str] | NotGiven = NOT_GIVEN,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        updated_at: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPage[Ledger]:
        """
        Get a list of ledgers.

        Args:
          id: If you have specific IDs to retrieve in bulk, you can pass them as query
              parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          updated_at: Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
              posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
              updated_at%5Bgt%5D=2000-01-01T12:00:00Z.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/ledgers",
            page=SyncPage[Ledger],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after_cursor": after_cursor,
                        "metadata": metadata,
                        "per_page": per_page,
                        "updated_at": updated_at,
                    },
                    ledger_list_params.LedgerListParams,
                ),
            ),
            model=Ledger,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Ledger:
        """
        Delete a ledger.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/api/ledgers/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Ledger,
        )


class AsyncLedgers(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLedgersWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLedgersWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLedgersWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncLedgersWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Ledger:
        """
        Create a ledger.

        Args:
          name: The name of the ledger.

          description: An optional free-form description for internal use.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/ledgers",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "metadata": metadata,
                },
                ledger_create_params.LedgerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Ledger,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Ledger:
        """
        Get details on a single ledger.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/ledgers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Ledger,
        )

    async def update(
        self,
        id: str,
        *,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Ledger:
        """
        Update the details of a ledger.

        Args:
          description: An optional free-form description for internal use.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          name: The name of the ledger.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/ledgers/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "metadata": metadata,
                    "name": name,
                },
                ledger_update_params.LedgerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Ledger,
        )

    def list(
        self,
        *,
        id: List[str] | NotGiven = NOT_GIVEN,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        updated_at: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Ledger, AsyncPage[Ledger]]:
        """
        Get a list of ledgers.

        Args:
          id: If you have specific IDs to retrieve in bulk, you can pass them as query
              parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          updated_at: Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
              posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
              updated_at%5Bgt%5D=2000-01-01T12:00:00Z.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/ledgers",
            page=AsyncPage[Ledger],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after_cursor": after_cursor,
                        "metadata": metadata,
                        "per_page": per_page,
                        "updated_at": updated_at,
                    },
                    ledger_list_params.LedgerListParams,
                ),
            ),
            model=Ledger,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Ledger:
        """
        Delete a ledger.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/api/ledgers/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Ledger,
        )


class LedgersWithRawResponse:
    def __init__(self, ledgers: Ledgers) -> None:
        self._ledgers = ledgers

        self.create = _legacy_response.to_raw_response_wrapper(
            ledgers.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            ledgers.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            ledgers.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            ledgers.list,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            ledgers.delete,
        )


class AsyncLedgersWithRawResponse:
    def __init__(self, ledgers: AsyncLedgers) -> None:
        self._ledgers = ledgers

        self.create = _legacy_response.async_to_raw_response_wrapper(
            ledgers.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            ledgers.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            ledgers.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            ledgers.list,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            ledgers.delete,
        )


class LedgersWithStreamingResponse:
    def __init__(self, ledgers: Ledgers) -> None:
        self._ledgers = ledgers

        self.create = to_streamed_response_wrapper(
            ledgers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ledgers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            ledgers.update,
        )
        self.list = to_streamed_response_wrapper(
            ledgers.list,
        )
        self.delete = to_streamed_response_wrapper(
            ledgers.delete,
        )


class AsyncLedgersWithStreamingResponse:
    def __init__(self, ledgers: AsyncLedgers) -> None:
        self._ledgers = ledgers

        self.create = async_to_streamed_response_wrapper(
            ledgers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ledgers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            ledgers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ledgers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            ledgers.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import (
    AccountCollectionFlow,
)
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAccountCollectionFlows:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        account_collection_flow = client.account_collection_flows.create(
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            payment_types=["string"],
        )
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        account_collection_flow = client.account_collection_flows.create(
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            payment_types=["string"],
            receiving_countries=["USA"],
        )
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.account_collection_flows.with_raw_response.create(
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            payment_types=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_collection_flow = response.parse()
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.account_collection_flows.with_streaming_response.create(
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            payment_types=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_collection_flow = response.parse()
            assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        account_collection_flow = client.account_collection_flows.retrieve(
            "id",
        )
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.account_collection_flows.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_collection_flow = response.parse()
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.account_collection_flows.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_collection_flow = response.parse()
            assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.account_collection_flows.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        account_collection_flow = client.account_collection_flows.update(
            id="id",
            status="cancelled",
        )
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.account_collection_flows.with_raw_response.update(
            id="id",
            status="cancelled",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_collection_flow = response.parse()
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.account_collection_flows.with_streaming_response.update(
            id="id",
            status="cancelled",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_collection_flow = response.parse()
            assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.account_collection_flows.with_raw_response.update(
                id="",
                status="cancelled",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        account_collection_flow = client.account_collection_flows.list()
        assert_matches_type(SyncPage[AccountCollectionFlow], account_collection_flow, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        account_collection_flow = client.account_collection_flows.list(
            after_cursor="after_cursor",
            client_token="client_token",
            counterparty_id="counterparty_id",
            external_account_id="external_account_id",
            per_page=0,
            status="status",
        )
        assert_matches_type(SyncPage[AccountCollectionFlow], account_collection_flow, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.account_collection_flows.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_collection_flow = response.parse()
        assert_matches_type(SyncPage[AccountCollectionFlow], account_collection_flow, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.account_collection_flows.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_collection_flow = response.parse()
            assert_matches_type(SyncPage[AccountCollectionFlow], account_collection_flow, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAccountCollectionFlows:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        account_collection_flow = await async_client.account_collection_flows.create(
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            payment_types=["string"],
        )
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        account_collection_flow = await async_client.account_collection_flows.create(
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            payment_types=["string"],
            receiving_countries=["USA"],
        )
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.account_collection_flows.with_raw_response.create(
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            payment_types=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_collection_flow = response.parse()
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.account_collection_flows.with_streaming_response.create(
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            payment_types=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_collection_flow = await response.parse()
            assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        account_collection_flow = await async_client.account_collection_flows.retrieve(
            "id",
        )
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.account_collection_flows.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_collection_flow = response.parse()
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.account_collection_flows.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_collection_flow = await response.parse()
            assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.account_collection_flows.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        account_collection_flow = await async_client.account_collection_flows.update(
            id="id",
            status="cancelled",
        )
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.account_collection_flows.with_raw_response.update(
            id="id",
            status="cancelled",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_collection_flow = response.parse()
        assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.account_collection_flows.with_streaming_response.update(
            id="id",
            status="cancelled",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_collection_flow = await response.parse()
            assert_matches_type(AccountCollectionFlow, account_collection_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.account_collection_flows.with_raw_response.update(
                id="",
                status="cancelled",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        account_collection_flow = await async_client.account_collection_flows.list()
        assert_matches_type(AsyncPage[AccountCollectionFlow], account_collection_flow, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        account_collection_flow = await async_client.account_collection_flows.list(
            after_cursor="after_cursor",
            client_token="client_token",
            counterparty_id="counterparty_id",
            external_account_id="external_account_id",
            per_page=0,
            status="status",
        )
        assert_matches_type(AsyncPage[AccountCollectionFlow], account_collection_flow, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.account_collection_flows.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_collection_flow = response.parse()
        assert_matches_type(AsyncPage[AccountCollectionFlow], account_collection_flow, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.account_collection_flows.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_collection_flow = await response.parse()
            assert_matches_type(AsyncPage[AccountCollectionFlow], account_collection_flow, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .balance_reports import (
    BalanceReports,
    AsyncBalanceReports,
    BalanceReportsWithRawResponse,
    AsyncBalanceReportsWithRawResponse,
    BalanceReportsWithStreamingResponse,
    AsyncBalanceReportsWithStreamingResponse,
)
from .internal_accounts import (
    InternalAccounts,
    AsyncInternalAccounts,
    InternalAccountsWithRawResponse,
    AsyncInternalAccountsWithRawResponse,
    InternalAccountsWithStreamingResponse,
    AsyncInternalAccountsWithStreamingResponse,
)

__all__ = [
    "BalanceReports",
    "AsyncBalanceReports",
    "BalanceReportsWithRawResponse",
    "AsyncBalanceReportsWithRawResponse",
    "BalanceReportsWithStreamingResponse",
    "AsyncBalanceReportsWithStreamingResponse",
    "InternalAccounts",
    "AsyncInternalAccounts",
    "InternalAccountsWithRawResponse",
    "AsyncInternalAccountsWithRawResponse",
    "InternalAccountsWithStreamingResponse",
    "AsyncInternalAccountsWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared.currency import Currency
from .expected_payment_type import ExpectedPaymentType
from .reconciliation_rule_param import ReconciliationRuleParam

__all__ = ["ExpectedPaymentUpdateParams"]


class ExpectedPaymentUpdateParams(TypedDict, total=False):
    amount_lower_bound: Optional[int]
    """The lowest amount this expected payment may be equal to.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    amount_upper_bound: Optional[int]
    """The highest amount this expected payment may be equal to.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    counterparty_id: Optional[str]
    """The ID of the counterparty you expect for this payment."""

    currency: Optional[Currency]
    """Must conform to ISO 4217. Defaults to the currency of the internal account."""

    date_lower_bound: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """The earliest date the payment may come in. Format: yyyy-mm-dd"""

    date_upper_bound: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """The latest date the payment may come in. Format: yyyy-mm-dd"""

    description: Optional[str]
    """An optional description for internal use."""

    direction: Optional[Literal["credit", "debit"]]
    """One of credit or debit.

    When you are receiving money, use credit. When you are being charged, use debit.
    """

    internal_account_id: Optional[str]
    """The ID of the Internal Account for the expected payment."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    reconciliation_filters: Optional[object]
    """The reconciliation filters you have for this payment."""

    reconciliation_groups: Optional[object]
    """The reconciliation groups you have for this payment."""

    reconciliation_rule_variables: Optional[Iterable[ReconciliationRuleParam]]
    """An array of reconciliation rule variables for this payment."""

    remittance_information: Optional[str]
    """For `ach`, this field will be passed through on an addenda record.

    For `wire` payments the field will be passed through as the "Originator to
    Beneficiary Information", also known as OBI or Fedwire tag 6000.
    """

    statement_descriptor: Optional[str]
    """The statement description you expect to see on the transaction.

    For ACH payments, this will be the full line item passed from the bank. For wire
    payments, this will be the OBI field on the wire. For check payments, this will
    be the memo field.
    """

    status: Optional[Literal["reconciled"]]
    """
    The Expected Payment's status can be updated from partially_reconciled to
    reconciled.
    """

    type: Optional[ExpectedPaymentType]
    """
    One of: ach, au_becs, bacs, book, check, eft, interac, provxchange, rtp, sen,
    sepa, signet, wire.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .version_list_params import VersionListParams as VersionListParams
from .ledger_transaction_version import LedgerTransactionVersion as LedgerTransactionVersion



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import TypedDict

__all__ = ["LedgerListParams"]


class LedgerListParams(TypedDict, total=False):
    id: List[str]
    """
    If you have specific IDs to retrieve in bulk, you can pass them as query
    parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.
    """

    after_cursor: Optional[str]

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    per_page: int

    updated_at: Dict[str, Union[str, datetime]]
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
    posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
    updated_at%5Bgt%5D=2000-01-01T12:00:00Z.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LineItemCreateParams"]


class LineItemCreateParams(TypedDict, total=False):
    amount: Required[int]
    """If a matching object exists in Modern Treasury, `amount` will be populated.

    Value in specified currency's smallest unit (taken from parent Transaction).
    """

    expected_payment_id: Required[str]
    """The ID of the reconciled Expected Payment, otherwise `null`."""

    transaction_id: Required[str]
    """The ID of the parent transaction."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PaymentFlowUpdateParams"]


class PaymentFlowUpdateParams(TypedDict, total=False):
    status: Required[Literal["cancelled"]]
    """Required.

    The updated status of the payment flow. Can only be used to mark a flow as
    `cancelled`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .balance_report import BalanceReport as BalanceReport
from .balance_report_list_params import BalanceReportListParams as BalanceReportListParams
from .balance_report_create_params import BalanceReportCreateParams as BalanceReportCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .invoices import (
    Invoices,
    AsyncInvoices,
    InvoicesWithRawResponse,
    AsyncInvoicesWithRawResponse,
    InvoicesWithStreamingResponse,
    AsyncInvoicesWithStreamingResponse,
)
from .line_items import (
    LineItems,
    AsyncLineItems,
    LineItemsWithRawResponse,
    AsyncLineItemsWithRawResponse,
    LineItemsWithStreamingResponse,
    AsyncLineItemsWithStreamingResponse,
)

__all__ = [
    "LineItems",
    "AsyncLineItems",
    "LineItemsWithRawResponse",
    "AsyncLineItemsWithRawResponse",
    "LineItemsWithStreamingResponse",
    "AsyncLineItemsWithStreamingResponse",
    "Invoices",
    "AsyncInvoices",
    "InvoicesWithRawResponse",
    "AsyncInvoicesWithRawResponse",
    "InvoicesWithStreamingResponse",
    "AsyncInvoicesWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import (
    ConnectionLegalEntity,
)
from modern_treasury._utils import parse_date, parse_datetime
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestConnectionLegalEntities:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        connection_legal_entity = client.connection_legal_entities.create(
            connection_id="connection_id",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        connection_legal_entity = client.connection_legal_entities.create(
            connection_id="connection_id",
            legal_entity={
                "addresses": [
                    {
                        "country": "country",
                        "line1": "line1",
                        "locality": "locality",
                        "postal_code": "postal_code",
                        "region": "region",
                        "address_types": ["business"],
                        "line2": "line2",
                    }
                ],
                "bank_settings": {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "backup_withholding_percentage": 0,
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "enable_backup_withholding": True,
                    "live_mode": True,
                    "object": "object",
                    "privacy_opt_out": True,
                    "regulation_o": True,
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                },
                "business_name": "business_name",
                "citizenship_country": "citizenship_country",
                "date_formed": parse_date("2019-12-27"),
                "date_of_birth": parse_date("2019-12-27"),
                "doing_business_as_names": ["string"],
                "email": "email",
                "first_name": "first_name",
                "identifications": [
                    {
                        "id_number": "id_number",
                        "id_type": "ar_cuil",
                        "issuing_country": "issuing_country",
                    }
                ],
                "last_name": "last_name",
                "legal_entity_associations": [
                    {
                        "relationship_types": ["beneficial_owner"],
                        "child_legal_entity": {
                            "addresses": [
                                {
                                    "country": "country",
                                    "line1": "line1",
                                    "locality": "locality",
                                    "postal_code": "postal_code",
                                    "region": "region",
                                    "address_types": ["business"],
                                    "line2": "line2",
                                }
                            ],
                            "bank_settings": {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "backup_withholding_percentage": 0,
                                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "enable_backup_withholding": True,
                                "live_mode": True,
                                "object": "object",
                                "privacy_opt_out": True,
                                "regulation_o": True,
                                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            },
                            "business_name": "business_name",
                            "citizenship_country": "citizenship_country",
                            "date_formed": parse_date("2019-12-27"),
                            "date_of_birth": parse_date("2019-12-27"),
                            "doing_business_as_names": ["string"],
                            "email": "email",
                            "first_name": "first_name",
                            "identifications": [
                                {
                                    "id_number": "id_number",
                                    "id_type": "ar_cuil",
                                    "issuing_country": "issuing_country",
                                }
                            ],
                            "last_name": "last_name",
                            "legal_entity_type": "business",
                            "legal_structure": "corporation",
                            "metadata": {
                                "key": "value",
                                "foo": "bar",
                                "modern": "treasury",
                            },
                            "middle_name": "middle_name",
                            "phone_numbers": [{"phone_number": "phone_number"}],
                            "politically_exposed_person": True,
                            "preferred_name": "preferred_name",
                            "prefix": "prefix",
                            "risk_rating": "low",
                            "suffix": "suffix",
                            "wealth_and_employment_details": {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "annual_income": 0,
                                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "employer_country": "employer_country",
                                "employer_name": "employer_name",
                                "employer_state": "employer_state",
                                "employment_status": "employed",
                                "income_country": "income_country",
                                "income_source": "family_support",
                                "income_state": "income_state",
                                "industry": "accounting",
                                "live_mode": True,
                                "object": "object",
                                "occupation": "consulting",
                                "source_of_funds": "alimony",
                                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "wealth_source": "business_sale",
                            },
                            "website": "website",
                        },
                        "child_legal_entity_id": "child_legal_entity_id",
                        "ownership_percentage": 0,
                        "title": "title",
                    }
                ],
                "legal_entity_type": "business",
                "legal_structure": "corporation",
                "metadata": {
                    "key": "value",
                    "foo": "bar",
                    "modern": "treasury",
                },
                "middle_name": "middle_name",
                "phone_numbers": [{"phone_number": "phone_number"}],
                "politically_exposed_person": True,
                "preferred_name": "preferred_name",
                "prefix": "prefix",
                "risk_rating": "low",
                "suffix": "suffix",
                "wealth_and_employment_details": {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "annual_income": 0,
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "employer_country": "employer_country",
                    "employer_name": "employer_name",
                    "employer_state": "employer_state",
                    "employment_status": "employed",
                    "income_country": "income_country",
                    "income_source": "family_support",
                    "income_state": "income_state",
                    "industry": "accounting",
                    "live_mode": True,
                    "object": "object",
                    "occupation": "consulting",
                    "source_of_funds": "alimony",
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "wealth_source": "business_sale",
                },
                "website": "website",
            },
            legal_entity_id="legal_entity_id",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.connection_legal_entities.with_raw_response.create(
            connection_id="connection_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection_legal_entity = response.parse()
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.connection_legal_entities.with_streaming_response.create(
            connection_id="connection_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection_legal_entity = response.parse()
            assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        connection_legal_entity = client.connection_legal_entities.retrieve(
            "id",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.connection_legal_entities.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection_legal_entity = response.parse()
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.connection_legal_entities.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection_legal_entity = response.parse()
            assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.connection_legal_entities.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        connection_legal_entity = client.connection_legal_entities.update(
            id="id",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: ModernTreasury) -> None:
        connection_legal_entity = client.connection_legal_entities.update(
            id="id",
            status="processing",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.connection_legal_entities.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection_legal_entity = response.parse()
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.connection_legal_entities.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection_legal_entity = response.parse()
            assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.connection_legal_entities.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        connection_legal_entity = client.connection_legal_entities.list()
        assert_matches_type(SyncPage[ConnectionLegalEntity], connection_legal_entity, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        connection_legal_entity = client.connection_legal_entities.list(
            after_cursor="after_cursor",
            connection_id="connection_id",
            legal_entity_id="legal_entity_id",
            per_page=0,
            status="completed",
        )
        assert_matches_type(SyncPage[ConnectionLegalEntity], connection_legal_entity, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.connection_legal_entities.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection_legal_entity = response.parse()
        assert_matches_type(SyncPage[ConnectionLegalEntity], connection_legal_entity, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.connection_legal_entities.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection_legal_entity = response.parse()
            assert_matches_type(SyncPage[ConnectionLegalEntity], connection_legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncConnectionLegalEntities:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        connection_legal_entity = await async_client.connection_legal_entities.create(
            connection_id="connection_id",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        connection_legal_entity = await async_client.connection_legal_entities.create(
            connection_id="connection_id",
            legal_entity={
                "addresses": [
                    {
                        "country": "country",
                        "line1": "line1",
                        "locality": "locality",
                        "postal_code": "postal_code",
                        "region": "region",
                        "address_types": ["business"],
                        "line2": "line2",
                    }
                ],
                "bank_settings": {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "backup_withholding_percentage": 0,
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "enable_backup_withholding": True,
                    "live_mode": True,
                    "object": "object",
                    "privacy_opt_out": True,
                    "regulation_o": True,
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                },
                "business_name": "business_name",
                "citizenship_country": "citizenship_country",
                "date_formed": parse_date("2019-12-27"),
                "date_of_birth": parse_date("2019-12-27"),
                "doing_business_as_names": ["string"],
                "email": "email",
                "first_name": "first_name",
                "identifications": [
                    {
                        "id_number": "id_number",
                        "id_type": "ar_cuil",
                        "issuing_country": "issuing_country",
                    }
                ],
                "last_name": "last_name",
                "legal_entity_associations": [
                    {
                        "relationship_types": ["beneficial_owner"],
                        "child_legal_entity": {
                            "addresses": [
                                {
                                    "country": "country",
                                    "line1": "line1",
                                    "locality": "locality",
                                    "postal_code": "postal_code",
                                    "region": "region",
                                    "address_types": ["business"],
                                    "line2": "line2",
                                }
                            ],
                            "bank_settings": {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "backup_withholding_percentage": 0,
                                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "enable_backup_withholding": True,
                                "live_mode": True,
                                "object": "object",
                                "privacy_opt_out": True,
                                "regulation_o": True,
                                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            },
                            "business_name": "business_name",
                            "citizenship_country": "citizenship_country",
                            "date_formed": parse_date("2019-12-27"),
                            "date_of_birth": parse_date("2019-12-27"),
                            "doing_business_as_names": ["string"],
                            "email": "email",
                            "first_name": "first_name",
                            "identifications": [
                                {
                                    "id_number": "id_number",
                                    "id_type": "ar_cuil",
                                    "issuing_country": "issuing_country",
                                }
                            ],
                            "last_name": "last_name",
                            "legal_entity_type": "business",
                            "legal_structure": "corporation",
                            "metadata": {
                                "key": "value",
                                "foo": "bar",
                                "modern": "treasury",
                            },
                            "middle_name": "middle_name",
                            "phone_numbers": [{"phone_number": "phone_number"}],
                            "politically_exposed_person": True,
                            "preferred_name": "preferred_name",
                            "prefix": "prefix",
                            "risk_rating": "low",
                            "suffix": "suffix",
                            "wealth_and_employment_details": {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "annual_income": 0,
                                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "employer_country": "employer_country",
                                "employer_name": "employer_name",
                                "employer_state": "employer_state",
                                "employment_status": "employed",
                                "income_country": "income_country",
                                "income_source": "family_support",
                                "income_state": "income_state",
                                "industry": "accounting",
                                "live_mode": True,
                                "object": "object",
                                "occupation": "consulting",
                                "source_of_funds": "alimony",
                                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "wealth_source": "business_sale",
                            },
                            "website": "website",
                        },
                        "child_legal_entity_id": "child_legal_entity_id",
                        "ownership_percentage": 0,
                        "title": "title",
                    }
                ],
                "legal_entity_type": "business",
                "legal_structure": "corporation",
                "metadata": {
                    "key": "value",
                    "foo": "bar",
                    "modern": "treasury",
                },
                "middle_name": "middle_name",
                "phone_numbers": [{"phone_number": "phone_number"}],
                "politically_exposed_person": True,
                "preferred_name": "preferred_name",
                "prefix": "prefix",
                "risk_rating": "low",
                "suffix": "suffix",
                "wealth_and_employment_details": {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "annual_income": 0,
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "employer_country": "employer_country",
                    "employer_name": "employer_name",
                    "employer_state": "employer_state",
                    "employment_status": "employed",
                    "income_country": "income_country",
                    "income_source": "family_support",
                    "income_state": "income_state",
                    "industry": "accounting",
                    "live_mode": True,
                    "object": "object",
                    "occupation": "consulting",
                    "source_of_funds": "alimony",
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "wealth_source": "business_sale",
                },
                "website": "website",
            },
            legal_entity_id="legal_entity_id",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.connection_legal_entities.with_raw_response.create(
            connection_id="connection_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection_legal_entity = response.parse()
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.connection_legal_entities.with_streaming_response.create(
            connection_id="connection_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection_legal_entity = await response.parse()
            assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        connection_legal_entity = await async_client.connection_legal_entities.retrieve(
            "id",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.connection_legal_entities.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection_legal_entity = response.parse()
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.connection_legal_entities.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection_legal_entity = await response.parse()
            assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.connection_legal_entities.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        connection_legal_entity = await async_client.connection_legal_entities.update(
            id="id",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        connection_legal_entity = await async_client.connection_legal_entities.update(
            id="id",
            status="processing",
        )
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.connection_legal_entities.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection_legal_entity = response.parse()
        assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.connection_legal_entities.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection_legal_entity = await response.parse()
            assert_matches_type(ConnectionLegalEntity, connection_legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.connection_legal_entities.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        connection_legal_entity = await async_client.connection_legal_entities.list()
        assert_matches_type(AsyncPage[ConnectionLegalEntity], connection_legal_entity, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        connection_legal_entity = await async_client.connection_legal_entities.list(
            after_cursor="after_cursor",
            connection_id="connection_id",
            legal_entity_id="legal_entity_id",
            per_page=0,
            status="completed",
        )
        assert_matches_type(AsyncPage[ConnectionLegalEntity], connection_legal_entity, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.connection_legal_entities.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        connection_legal_entity = response.parse()
        assert_matches_type(AsyncPage[ConnectionLegalEntity], connection_legal_entity, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.connection_legal_entities.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            connection_legal_entity = await response.parse()
            assert_matches_type(AsyncPage[ConnectionLegalEntity], connection_legal_entity, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime

import httpx

from .. import _legacy_response
from ..types import event_list_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from ..types.event import Event
from .._base_client import AsyncPaginator, make_request_options

__all__ = ["Events", "AsyncEvents"]


class Events(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return EventsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return EventsWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Event:
        """
        get event

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/events/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Event,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        entity_id: str | NotGiven = NOT_GIVEN,
        event_name: str | NotGiven = NOT_GIVEN,
        event_time_end: Union[str, datetime] | NotGiven = NOT_GIVEN,
        event_time_start: Union[str, datetime] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        resource: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPage[Event]:
        """
        list events

        Args:
          event_time_end: An inclusive upper bound for when the event occurred

          event_time_start: An inclusive lower bound for when the event occurred

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/events",
            page=SyncPage[Event],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "entity_id": entity_id,
                        "event_name": event_name,
                        "event_time_end": event_time_end,
                        "event_time_start": event_time_start,
                        "per_page": per_page,
                        "resource": resource,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=Event,
        )


class AsyncEvents(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncEventsWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Event:
        """
        get event

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/events/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Event,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        entity_id: str | NotGiven = NOT_GIVEN,
        event_name: str | NotGiven = NOT_GIVEN,
        event_time_end: Union[str, datetime] | NotGiven = NOT_GIVEN,
        event_time_start: Union[str, datetime] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        resource: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Event, AsyncPage[Event]]:
        """
        list events

        Args:
          event_time_end: An inclusive upper bound for when the event occurred

          event_time_start: An inclusive lower bound for when the event occurred

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/events",
            page=AsyncPage[Event],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "entity_id": entity_id,
                        "event_name": event_name,
                        "event_time_end": event_time_end,
                        "event_time_start": event_time_start,
                        "per_page": per_page,
                        "resource": resource,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=Event,
        )


class EventsWithRawResponse:
    def __init__(self, events: Events) -> None:
        self._events = events

        self.retrieve = _legacy_response.to_raw_response_wrapper(
            events.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            events.list,
        )


class AsyncEventsWithRawResponse:
    def __init__(self, events: AsyncEvents) -> None:
        self._events = events

        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            events.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            events.list,
        )


class EventsWithStreamingResponse:
    def __init__(self, events: Events) -> None:
        self._events = events

        self.retrieve = to_streamed_response_wrapper(
            events.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            events.list,
        )


class AsyncEventsWithStreamingResponse:
    def __init__(self, events: AsyncEvents) -> None:
        self._events = events

        self.retrieve = async_to_streamed_response_wrapper(
            events.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            events.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .event import Event as Event
from .ledger import Ledger as Ledger
from .shared import (
    Currency as Currency,
    AccountsType as AccountsType,
    AsyncResponse as AsyncResponse,
    TransactionDirection as TransactionDirection,
)
from .invoice import Invoice as Invoice
from .document import Document as Document
from .line_item import LineItem as LineItem
from .connection import Connection as Connection
from .paper_item import PaperItem as PaperItem
from .bulk_result import BulkResult as BulkResult
from .transaction import Transaction as Transaction
from .bulk_request import BulkRequest as BulkRequest
from .counterparty import Counterparty as Counterparty
from .ledger_entry import LedgerEntry as LedgerEntry
from .legal_entity import LegalEntity as LegalEntity
from .payment_flow import PaymentFlow as PaymentFlow
from .bank_settings import BankSettings as BankSettings
from .payment_order import PaymentOrder as PaymentOrder
from .ping_response import PingResponse as PingResponse
from .return_object import ReturnObject as ReturnObject
from .account_detail import AccountDetail as AccountDetail
from .ledger_account import LedgerAccount as LedgerAccount
from .routing_detail import RoutingDetail as RoutingDetail
from .virtual_account import VirtualAccount as VirtualAccount
from .expected_payment import ExpectedPayment as ExpectedPayment
from .external_account import ExternalAccount as ExternalAccount
from .internal_account import InternalAccount as InternalAccount
from .ledgerable_event import LedgerableEvent as LedgerableEvent
from .event_list_params import EventListParams as EventListParams
from .payment_reference import PaymentReference as PaymentReference
from .ledger_list_params import LedgerListParams as LedgerListParams
from .ledger_transaction import LedgerTransaction as LedgerTransaction
from .payment_order_type import PaymentOrderType as PaymentOrderType
from .return_list_params import ReturnListParams as ReturnListParams
from .bank_settings_param import BankSettingsParam as BankSettingsParam
from .invoice_list_params import InvoiceListParams as InvoiceListParams
from .reconciliation_rule import ReconciliationRule as ReconciliationRule
from .document_list_params import DocumentListParams as DocumentListParams
from .ledger_create_params import LedgerCreateParams as LedgerCreateParams
from .ledger_event_handler import LedgerEventHandler as LedgerEventHandler
from .ledger_update_params import LedgerUpdateParams as LedgerUpdateParams
from .return_create_params import ReturnCreateParams as ReturnCreateParams
from .expected_payment_type import ExpectedPaymentType as ExpectedPaymentType
from .external_account_type import ExternalAccountType as ExternalAccountType
from .invoice_create_params import InvoiceCreateParams as InvoiceCreateParams
from .invoice_update_params import InvoiceUpdateParams as InvoiceUpdateParams
from .line_item_list_params import LineItemListParams as LineItemListParams
from .payment_order_subtype import PaymentOrderSubtype as PaymentOrderSubtype
from .connection_list_params import ConnectionListParams as ConnectionListParams
from .document_create_params import DocumentCreateParams as DocumentCreateParams
from .foreign_exchange_quote import ForeignExchangeQuote as ForeignExchangeQuote
from .paper_item_list_params import PaperItemListParams as PaperItemListParams
from .account_collection_flow import AccountCollectionFlow as AccountCollectionFlow
from .bulk_result_list_params import BulkResultListParams as BulkResultListParams
from .connection_legal_entity import ConnectionLegalEntity as ConnectionLegalEntity
from .incoming_payment_detail import IncomingPaymentDetail as IncomingPaymentDetail
from .ledger_account_category import LedgerAccountCategory as LedgerAccountCategory
from .line_item_update_params import LineItemUpdateParams as LineItemUpdateParams
from .transaction_list_params import TransactionListParams as TransactionListParams
from .bulk_request_list_params import BulkRequestListParams as BulkRequestListParams
from .counterparty_list_params import CounterpartyListParams as CounterpartyListParams
from .ledger_entry_list_params import LedgerEntryListParams as LedgerEntryListParams
from .legal_entity_association import LegalEntityAssociation as LegalEntityAssociation
from .legal_entity_list_params import LegalEntityListParams as LegalEntityListParams
from .payment_flow_list_params import PaymentFlowListParams as PaymentFlowListParams
from .ledger_account_settlement import LedgerAccountSettlement as LedgerAccountSettlement
from .payment_order_list_params import PaymentOrderListParams as PaymentOrderListParams
from .reconciliation_rule_param import ReconciliationRuleParam as ReconciliationRuleParam
from .transaction_create_params import TransactionCreateParams as TransactionCreateParams
from .transaction_update_params import TransactionUpdateParams as TransactionUpdateParams
from .account_detail_list_params import AccountDetailListParams as AccountDetailListParams
from .bulk_request_create_params import BulkRequestCreateParams as BulkRequestCreateParams
from .counterparty_create_params import CounterpartyCreateParams as CounterpartyCreateParams
from .counterparty_update_params import CounterpartyUpdateParams as CounterpartyUpdateParams
from .ledger_account_list_params import LedgerAccountListParams as LedgerAccountListParams
from .ledger_entry_update_params import LedgerEntryUpdateParams as LedgerEntryUpdateParams
from .legal_entity_create_params import LegalEntityCreateParams as LegalEntityCreateParams
from .legal_entity_update_params import LegalEntityUpdateParams as LegalEntityUpdateParams
from .payment_flow_create_params import PaymentFlowCreateParams as PaymentFlowCreateParams
from .payment_flow_update_params import PaymentFlowUpdateParams as PaymentFlowUpdateParams
from .routing_detail_list_params import RoutingDetailListParams as RoutingDetailListParams
from .payment_order_create_params import PaymentOrderCreateParams as PaymentOrderCreateParams
from .payment_order_update_params import PaymentOrderUpdateParams as PaymentOrderUpdateParams
from .virtual_account_list_params import VirtualAccountListParams as VirtualAccountListParams
from .account_detail_create_params import AccountDetailCreateParams as AccountDetailCreateParams
from .expected_payment_list_params import ExpectedPaymentListParams as ExpectedPaymentListParams
from .external_account_list_params import ExternalAccountListParams as ExternalAccountListParams
from .internal_account_list_params import InternalAccountListParams as InternalAccountListParams
from .ledger_account_create_params import LedgerAccountCreateParams as LedgerAccountCreateParams
from .ledger_account_update_params import LedgerAccountUpdateParams as LedgerAccountUpdateParams
from .ledger_entry_retrieve_params import LedgerEntryRetrieveParams as LedgerEntryRetrieveParams
from .routing_detail_create_params import RoutingDetailCreateParams as RoutingDetailCreateParams
from .ledger_event_handler_variable import LedgerEventHandlerVariable as LedgerEventHandlerVariable
from .payment_reference_list_params import PaymentReferenceListParams as PaymentReferenceListParams
from .routing_number_lookup_request import RoutingNumberLookupRequest as RoutingNumberLookupRequest
from .virtual_account_create_params import VirtualAccountCreateParams as VirtualAccountCreateParams
from .virtual_account_update_params import VirtualAccountUpdateParams as VirtualAccountUpdateParams
from .wealth_and_employment_details import WealthAndEmploymentDetails as WealthAndEmploymentDetails
from .expected_payment_create_params import ExpectedPaymentCreateParams as ExpectedPaymentCreateParams
from .expected_payment_update_params import ExpectedPaymentUpdateParams as ExpectedPaymentUpdateParams
from .external_account_create_params import ExternalAccountCreateParams as ExternalAccountCreateParams
from .external_account_update_params import ExternalAccountUpdateParams as ExternalAccountUpdateParams
from .external_account_verify_params import ExternalAccountVerifyParams as ExternalAccountVerifyParams
from .internal_account_create_params import InternalAccountCreateParams as InternalAccountCreateParams
from .internal_account_update_params import InternalAccountUpdateParams as InternalAccountUpdateParams
from .ledger_account_balance_monitor import LedgerAccountBalanceMonitor as LedgerAccountBalanceMonitor
from .ledger_account_retrieve_params import LedgerAccountRetrieveParams as LedgerAccountRetrieveParams
from .ledger_transaction_list_params import LedgerTransactionListParams as LedgerTransactionListParams
from .ledgerable_event_create_params import LedgerableEventCreateParams as LedgerableEventCreateParams
from .external_account_verify_response import ExternalAccountVerifyResponse as ExternalAccountVerifyResponse
from .ledger_event_handler_list_params import LedgerEventHandlerListParams as LedgerEventHandlerListParams
from .ledger_transaction_create_params import LedgerTransactionCreateParams as LedgerTransactionCreateParams
from .ledger_transaction_update_params import LedgerTransactionUpdateParams as LedgerTransactionUpdateParams
from .payment_order_create_async_params import PaymentOrderCreateAsyncParams as PaymentOrderCreateAsyncParams
from .foreign_exchange_quote_list_params import ForeignExchangeQuoteListParams as ForeignExchangeQuoteListParams
from .ledger_event_handler_create_params import LedgerEventHandlerCreateParams as LedgerEventHandlerCreateParams
from .account_collection_flow_list_params import AccountCollectionFlowListParams as AccountCollectionFlowListParams
from .connection_legal_entity_list_params import ConnectionLegalEntityListParams as ConnectionLegalEntityListParams
from .counterparty_collect_account_params import CounterpartyCollectAccountParams as CounterpartyCollectAccountParams
from .incoming_payment_detail_list_params import IncomingPaymentDetailListParams as IncomingPaymentDetailListParams
from .ledger_account_category_list_params import LedgerAccountCategoryListParams as LedgerAccountCategoryListParams
from .ledger_event_handler_variable_param import LedgerEventHandlerVariableParam as LedgerEventHandlerVariableParam
from .wealth_and_employment_details_param import WealthAndEmploymentDetailsParam as WealthAndEmploymentDetailsParam
from .foreign_exchange_quote_create_params import ForeignExchangeQuoteCreateParams as ForeignExchangeQuoteCreateParams
from .account_collection_flow_create_params import (
    AccountCollectionFlowCreateParams as AccountCollectionFlowCreateParams,
)
from .account_collection_flow_update_params import (
    AccountCollectionFlowUpdateParams as AccountCollectionFlowUpdateParams,
)
from .connection_legal_entity_create_params import (
    ConnectionLegalEntityCreateParams as ConnectionLegalEntityCreateParams,
)
from .connection_legal_entity_update_params import (
    ConnectionLegalEntityUpdateParams as ConnectionLegalEntityUpdateParams,
)
from .counterparty_collect_account_response import (
    CounterpartyCollectAccountResponse as CounterpartyCollectAccountResponse,
)
from .incoming_payment_detail_update_params import (
    IncomingPaymentDetailUpdateParams as IncomingPaymentDetailUpdateParams,
)
from .ledger_account_category_create_params import (
    LedgerAccountCategoryCreateParams as LedgerAccountCategoryCreateParams,
)
from .ledger_account_category_update_params import (
    LedgerAccountCategoryUpdateParams as LedgerAccountCategoryUpdateParams,
)
from .ledger_account_settlement_list_params import (
    LedgerAccountSettlementListParams as LedgerAccountSettlementListParams,
)
from .ledger_account_statement_create_params import (
    LedgerAccountStatementCreateParams as LedgerAccountStatementCreateParams,
)
from .legal_entity_association_create_params import (
    LegalEntityAssociationCreateParams as LegalEntityAssociationCreateParams,
)
from .ledger_account_category_retrieve_params import (
    LedgerAccountCategoryRetrieveParams as LedgerAccountCategoryRetrieveParams,
)
from .ledger_account_settlement_create_params import (
    LedgerAccountSettlementCreateParams as LedgerAccountSettlementCreateParams,
)
from .ledger_account_settlement_update_params import (
    LedgerAccountSettlementUpdateParams as LedgerAccountSettlementUpdateParams,
)
from .ledger_account_statement_create_response import (
    LedgerAccountStatementCreateResponse as LedgerAccountStatementCreateResponse,
)
from .ledger_transaction_create_reversal_params import (
    LedgerTransactionCreateReversalParams as LedgerTransactionCreateReversalParams,
)
from .validation_validate_routing_number_params import (
    ValidationValidateRoutingNumberParams as ValidationValidateRoutingNumberParams,
)
from .ledger_account_balance_monitor_list_params import (
    LedgerAccountBalanceMonitorListParams as LedgerAccountBalanceMonitorListParams,
)
from .ledger_account_statement_retrieve_response import (
    LedgerAccountStatementRetrieveResponse as LedgerAccountStatementRetrieveResponse,
)
from .incoming_payment_detail_create_async_params import (
    IncomingPaymentDetailCreateAsyncParams as IncomingPaymentDetailCreateAsyncParams,
)
from .ledger_account_balance_monitor_create_params import (
    LedgerAccountBalanceMonitorCreateParams as LedgerAccountBalanceMonitorCreateParams,
)
from .ledger_account_balance_monitor_update_params import (
    LedgerAccountBalanceMonitorUpdateParams as LedgerAccountBalanceMonitorUpdateParams,
)
from .external_account_complete_verification_params import (
    ExternalAccountCompleteVerificationParams as ExternalAccountCompleteVerificationParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import (
    LedgerEventHandler,
)
from modern_treasury._utils import parse_datetime
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLedgerEventHandlers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        ledger_event_handler = client.ledger_event_handlers.create(
            ledger_transaction_template={
                "description": "My Ledger Transaction Template Description",
                "effective_at": "{{ledgerable_event.custom_data.effective_at}}",
                "ledger_entries": [
                    {
                        "amount": "amount",
                        "direction": "direction",
                        "ledger_account_id": "ledger_account_id",
                    }
                ],
                "status": "posted",
            },
            name="name",
        )
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        ledger_event_handler = client.ledger_event_handlers.create(
            ledger_transaction_template={
                "description": "My Ledger Transaction Template Description",
                "effective_at": "{{ledgerable_event.custom_data.effective_at}}",
                "ledger_entries": [
                    {
                        "amount": "amount",
                        "direction": "direction",
                        "ledger_account_id": "ledger_account_id",
                    }
                ],
                "status": "posted",
            },
            name="name",
            conditions={
                "field": "ledgerable_event.name",
                "operator": "equals",
                "value": "credit_card_swipe",
            },
            description="description",
            ledger_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            variables={
                "credit_account": {
                    "query": {
                        "field": "name",
                        "operator": "equals",
                        "value": "my_credit_account",
                    },
                    "type": "ledger_account",
                }
            },
        )
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.ledger_event_handlers.with_raw_response.create(
            ledger_transaction_template={
                "description": "My Ledger Transaction Template Description",
                "effective_at": "{{ledgerable_event.custom_data.effective_at}}",
                "ledger_entries": [
                    {
                        "amount": "amount",
                        "direction": "direction",
                        "ledger_account_id": "ledger_account_id",
                    }
                ],
                "status": "posted",
            },
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ledger_event_handler = response.parse()
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.ledger_event_handlers.with_streaming_response.create(
            ledger_transaction_template={
                "description": "My Ledger Transaction Template Description",
                "effective_at": "{{ledgerable_event.custom_data.effective_at}}",
                "ledger_entries": [
                    {
                        "amount": "amount",
                        "direction": "direction",
                        "ledger_account_id": "ledger_account_id",
                    }
                ],
                "status": "posted",
            },
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ledger_event_handler = response.parse()
            assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        ledger_event_handler = client.ledger_event_handlers.retrieve(
            "id",
        )
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.ledger_event_handlers.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ledger_event_handler = response.parse()
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.ledger_event_handlers.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ledger_event_handler = response.parse()
            assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.ledger_event_handlers.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        ledger_event_handler = client.ledger_event_handlers.list()
        assert_matches_type(SyncPage[LedgerEventHandler], ledger_event_handler, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        ledger_event_handler = client.ledger_event_handlers.list(
            after_cursor="after_cursor",
            created_at={"foo": parse_datetime("2019-12-27T18:11:19.117Z")},
            metadata={"foo": "string"},
            name="name",
            per_page=0,
        )
        assert_matches_type(SyncPage[LedgerEventHandler], ledger_event_handler, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.ledger_event_handlers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ledger_event_handler = response.parse()
        assert_matches_type(SyncPage[LedgerEventHandler], ledger_event_handler, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.ledger_event_handlers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ledger_event_handler = response.parse()
            assert_matches_type(SyncPage[LedgerEventHandler], ledger_event_handler, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: ModernTreasury) -> None:
        ledger_event_handler = client.ledger_event_handlers.delete(
            "id",
        )
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: ModernTreasury) -> None:
        response = client.ledger_event_handlers.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ledger_event_handler = response.parse()
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: ModernTreasury) -> None:
        with client.ledger_event_handlers.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ledger_event_handler = response.parse()
            assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.ledger_event_handlers.with_raw_response.delete(
                "",
            )


class TestAsyncLedgerEventHandlers:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        ledger_event_handler = await async_client.ledger_event_handlers.create(
            ledger_transaction_template={
                "description": "My Ledger Transaction Template Description",
                "effective_at": "{{ledgerable_event.custom_data.effective_at}}",
                "ledger_entries": [
                    {
                        "amount": "amount",
                        "direction": "direction",
                        "ledger_account_id": "ledger_account_id",
                    }
                ],
                "status": "posted",
            },
            name="name",
        )
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        ledger_event_handler = await async_client.ledger_event_handlers.create(
            ledger_transaction_template={
                "description": "My Ledger Transaction Template Description",
                "effective_at": "{{ledgerable_event.custom_data.effective_at}}",
                "ledger_entries": [
                    {
                        "amount": "amount",
                        "direction": "direction",
                        "ledger_account_id": "ledger_account_id",
                    }
                ],
                "status": "posted",
            },
            name="name",
            conditions={
                "field": "ledgerable_event.name",
                "operator": "equals",
                "value": "credit_card_swipe",
            },
            description="description",
            ledger_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            variables={
                "credit_account": {
                    "query": {
                        "field": "name",
                        "operator": "equals",
                        "value": "my_credit_account",
                    },
                    "type": "ledger_account",
                }
            },
        )
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.ledger_event_handlers.with_raw_response.create(
            ledger_transaction_template={
                "description": "My Ledger Transaction Template Description",
                "effective_at": "{{ledgerable_event.custom_data.effective_at}}",
                "ledger_entries": [
                    {
                        "amount": "amount",
                        "direction": "direction",
                        "ledger_account_id": "ledger_account_id",
                    }
                ],
                "status": "posted",
            },
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ledger_event_handler = response.parse()
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.ledger_event_handlers.with_streaming_response.create(
            ledger_transaction_template={
                "description": "My Ledger Transaction Template Description",
                "effective_at": "{{ledgerable_event.custom_data.effective_at}}",
                "ledger_entries": [
                    {
                        "amount": "amount",
                        "direction": "direction",
                        "ledger_account_id": "ledger_account_id",
                    }
                ],
                "status": "posted",
            },
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ledger_event_handler = await response.parse()
            assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        ledger_event_handler = await async_client.ledger_event_handlers.retrieve(
            "id",
        )
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.ledger_event_handlers.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ledger_event_handler = response.parse()
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.ledger_event_handlers.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ledger_event_handler = await response.parse()
            assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.ledger_event_handlers.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        ledger_event_handler = await async_client.ledger_event_handlers.list()
        assert_matches_type(AsyncPage[LedgerEventHandler], ledger_event_handler, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        ledger_event_handler = await async_client.ledger_event_handlers.list(
            after_cursor="after_cursor",
            created_at={"foo": parse_datetime("2019-12-27T18:11:19.117Z")},
            metadata={"foo": "string"},
            name="name",
            per_page=0,
        )
        assert_matches_type(AsyncPage[LedgerEventHandler], ledger_event_handler, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.ledger_event_handlers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ledger_event_handler = response.parse()
        assert_matches_type(AsyncPage[LedgerEventHandler], ledger_event_handler, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.ledger_event_handlers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ledger_event_handler = await response.parse()
            assert_matches_type(AsyncPage[LedgerEventHandler], ledger_event_handler, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncModernTreasury) -> None:
        ledger_event_handler = await async_client.ledger_event_handlers.delete(
            "id",
        )
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.ledger_event_handlers.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ledger_event_handler = response.parse()
        assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.ledger_event_handlers.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ledger_event_handler = await response.parse()
            assert_matches_type(LedgerEventHandler, ledger_event_handler, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.ledger_event_handlers.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .reversals import (
    Reversals,
    AsyncReversals,
    ReversalsWithRawResponse,
    AsyncReversalsWithRawResponse,
    ReversalsWithStreamingResponse,
    AsyncReversalsWithStreamingResponse,
)
from .payment_orders import (
    PaymentOrders,
    AsyncPaymentOrders,
    PaymentOrdersWithRawResponse,
    AsyncPaymentOrdersWithRawResponse,
    PaymentOrdersWithStreamingResponse,
    AsyncPaymentOrdersWithStreamingResponse,
)

__all__ = [
    "Reversals",
    "AsyncReversals",
    "ReversalsWithRawResponse",
    "AsyncReversalsWithRawResponse",
    "ReversalsWithStreamingResponse",
    "AsyncReversalsWithStreamingResponse",
    "PaymentOrders",
    "AsyncPaymentOrders",
    "PaymentOrdersWithRawResponse",
    "AsyncPaymentOrdersWithRawResponse",
    "PaymentOrdersWithStreamingResponse",
    "AsyncPaymentOrdersWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BankSettingsParam"]


class BankSettingsParam(TypedDict, total=False):
    id: Required[str]

    backup_withholding_percentage: Required[Optional[int]]
    """The percentage of backup withholding to apply to the legal entity."""

    created_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    discarded_at: Required[Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]]

    enable_backup_withholding: Required[Optional[bool]]
    """Whether backup withholding is enabled.

    See more here -
    https://www.irs.gov/businesses/small-businesses-self-employed/backup-withholding.
    """

    live_mode: Required[bool]
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: Required[str]

    privacy_opt_out: Required[Optional[bool]]
    """Cross River Bank specific setting to opt out of privacy policy."""

    regulation_o: Required[Optional[bool]]
    """
    It covers, among other types of insider loans, extensions of credit by a member
    bank to an executive officer, director, or principal shareholder of the member
    bank; a bank holding company of which the member bank is a subsidiary; and any
    other subsidiary of that bank holding company.
    """

    updated_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date, datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["LedgerAccountCategoryRetrieveParams", "Balances"]


class LedgerAccountCategoryRetrieveParams(TypedDict, total=False):
    balances: Balances
    """
    For example, if you want the balances as of a particular time (ISO8601), the
    encoded query string would be `balances%5Beffective_at%5D=2000-12-31T12:00:00Z`.
    The balances as of a time are inclusive of entries with that exact time.
    """


class Balances(TypedDict, total=False):
    as_of_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]

    effective_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["WealthAndEmploymentDetails"]


class WealthAndEmploymentDetails(BaseModel):
    id: str

    annual_income: Optional[int] = None
    """The annual income of the individual."""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    employer_country: Optional[str] = None
    """The country in which the employer is located."""

    employer_name: Optional[str] = None
    """The name of the employer."""

    employer_state: Optional[str] = None
    """The state in which the employer is located."""

    employment_status: Optional[Literal["employed", "retired", "self_employed", "student", "unemployed"]] = None
    """The employment status of the individual."""

    income_country: Optional[str] = None
    """The country in which the individual's income is earned."""

    income_source: Optional[
        Literal[
            "family_support",
            "government_benefits",
            "inheritance",
            "investments",
            "rental_income",
            "retirement",
            "salary",
            "self_employed",
        ]
    ] = None
    """The source of the individual's income."""

    income_state: Optional[str] = None
    """The state in which the individual's income is earned."""

    industry: Optional[
        Literal[
            "accounting",
            "agriculture",
            "automotive",
            "chemical_manufacturing",
            "construction",
            "educational_medical",
            "food_service",
            "finance",
            "gasoline",
            "health_stores",
            "laundry",
            "maintenance",
            "manufacturing",
            "merchant_wholesale",
            "mining",
            "performing_arts",
            "professional_non_legal",
            "public_administration",
            "publishing",
            "real_estate",
            "recreation_gambling",
            "religious_charity",
            "rental_services",
            "retail_clothing",
            "retail_electronics",
            "retail_food",
            "retail_furnishing",
            "retail_home",
            "retail_non_store",
            "retail_sporting",
            "transportation",
            "travel",
            "utilities",
        ]
    ] = None
    """The industry of the individual."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    occupation: Optional[
        Literal[
            "consulting",
            "executive",
            "finance_accounting",
            "food_services",
            "government",
            "healthcare",
            "legal_services",
            "manufacturing",
            "other",
            "sales",
            "science_engineering",
            "technology",
        ]
    ] = None
    """The occupation of the individual."""

    source_of_funds: Optional[
        Literal[
            "alimony",
            "annuity",
            "business_owner",
            "general_employee",
            "government_benefits",
            "homemaker",
            "inheritance_gift",
            "investment",
            "legal_settlement",
            "lottery",
            "real_estate",
            "retired",
            "retirement",
            "salary",
            "self_employed",
            "senior_executive",
            "trust_income",
        ]
    ] = None
    """The source of the individual's funds."""

    updated_at: datetime

    wealth_source: Optional[
        Literal[
            "business_sale",
            "family_support",
            "government_benefits",
            "inheritance",
            "investments",
            "other",
            "rental_income",
            "retirement",
            "salary",
            "self_employed",
        ]
    ] = None
    """The source of the individual's wealth."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .account_detail import AccountDetail
from .routing_detail import RoutingDetail
from .external_account_type import ExternalAccountType

__all__ = ["ExternalAccount", "ContactDetails", "ContactDetail", "PartyAddress"]


class ContactDetail(BaseModel):
    id: str

    contact_identifier: str

    contact_identifier_type: Literal["email", "phone_number", "website"]

    created_at: datetime

    discarded_at: Optional[datetime] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    updated_at: datetime


ContactDetails = ContactDetail
"""This type is deprecated and will be removed in a future release.

Please use ContactDetail instead.
"""


class PartyAddress(BaseModel):
    id: str

    country: Optional[str] = None
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    created_at: datetime

    line1: Optional[str] = None

    line2: Optional[str] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    locality: Optional[str] = None
    """Locality or City."""

    object: str

    postal_code: Optional[str] = None
    """The postal code of the address."""

    region: Optional[str] = None
    """Region or State."""

    updated_at: datetime


class ExternalAccount(BaseModel):
    id: str

    account_details: List[AccountDetail]

    account_type: ExternalAccountType
    """Can be `checking`, `savings` or `other`."""

    contact_details: List[ContactDetail]

    counterparty_id: Optional[str] = None

    created_at: datetime

    discarded_at: Optional[datetime] = None

    ledger_account_id: Optional[str] = None
    """
    If the external account links to a ledger account in Modern Treasury, the id of
    the ledger account will be populated here.
    """

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: Optional[str] = None
    """A nickname for the external account.

    This is only for internal usage and won't affect any payments
    """

    object: str

    party_address: Optional[PartyAddress] = None
    """The address associated with the owner or `null`."""

    party_name: str
    """The legal name of the entity which owns the account."""

    party_type: Optional[Literal["business", "individual"]] = None
    """Either `individual` or `business`."""

    routing_details: List[RoutingDetail]

    updated_at: datetime

    verification_source: Optional[Literal["ach_prenote", "microdeposits", "plaid"]] = None

    verification_status: Literal["pending_verification", "unverified", "verified"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .currency import Currency as Currency
from .accounts_type import AccountsType as AccountsType
from .transaction_direction import TransactionDirection as TransactionDirection



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared.currency import Currency

__all__ = ["ReconciliationRuleParam"]


class ReconciliationRuleParam(TypedDict, total=False):
    amount_lower_bound: Required[int]
    """The lowest amount this expected payment may be equal to.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    amount_upper_bound: Required[int]
    """The highest amount this expected payment may be equal to.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    direction: Required[Literal["credit", "debit"]]
    """One of credit or debit.

    When you are receiving money, use credit. When you are being charged, use debit.
    """

    internal_account_id: Required[str]
    """The ID of the Internal Account for the expected payment"""

    counterparty_id: Optional[str]
    """The ID of the counterparty you expect for this payment"""

    currency: Currency
    """Must conform to ISO 4217. Defaults to the currency of the internal account"""

    custom_identifiers: Optional[Dict[str, str]]
    """A hash of custom identifiers for this payment"""

    date_lower_bound: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """The earliest date the payment may come in. Format is yyyy-mm-dd"""

    date_upper_bound: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """The latest date the payment may come in. Format is yyyy-mm-dd"""

    type: Optional[
        Literal[
            "ach",
            "au_becs",
            "bacs",
            "book",
            "card",
            "chats",
            "check",
            "cross_border",
            "dk_nets",
            "eft",
            "hu_ics",
            "interac",
            "masav",
            "mx_ccen",
            "neft",
            "nics",
            "nz_becs",
            "pl_elixir",
            "provxchange",
            "ro_sent",
            "rtp",
            "se_bankgirot",
            "sen",
            "sepa",
            "sg_giro",
            "sic",
            "signet",
            "sknbi",
            "wire",
            "zengin",
        ]
    ]
    """
    One of ach, au_becs, bacs, book, check, eft, interac, provxchange, rtp, sen,
    sepa, signet wire
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from ... import _legacy_response
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ...pagination import SyncPage, AsyncPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.payment_orders import reversal_list_params, reversal_create_params
from ...types.payment_orders.reversal import Reversal

__all__ = ["Reversals", "AsyncReversals"]


class Reversals(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ReversalsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return ReversalsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReversalsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return ReversalsWithStreamingResponse(self)

    def create(
        self,
        payment_order_id: str,
        *,
        reason: Literal[
            "duplicate",
            "incorrect_amount",
            "incorrect_receiving_account",
            "date_earlier_than_intended",
            "date_later_than_intended",
        ],
        ledger_transaction: reversal_create_params.LedgerTransaction | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Reversal:
        """
        Create a reversal for a payment order.

        Args:
          payment_order_id: The id of the payment order being reversed.

          reason: The reason for the reversal. Must be one of `duplicate`, `incorrect_amount`,
              `incorrect_receiving_account`, `date_earlier_than_intended`,
              `date_later_than_intended`.

          ledger_transaction: Specifies a ledger transaction object that will be created with the reversal. If
              the ledger transaction cannot be created, then the reversal creation will fail.
              The resulting ledger transaction will mirror the status of the reversal.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not payment_order_id:
            raise ValueError(f"Expected a non-empty value for `payment_order_id` but received {payment_order_id!r}")
        return self._post(
            f"/api/payment_orders/{payment_order_id}/reversals",
            body=maybe_transform(
                {
                    "reason": reason,
                    "ledger_transaction": ledger_transaction,
                    "metadata": metadata,
                },
                reversal_create_params.ReversalCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Reversal,
        )

    def retrieve(
        self,
        reversal_id: str,
        *,
        payment_order_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Reversal:
        """
        Get details on a single reversal of a payment order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not payment_order_id:
            raise ValueError(f"Expected a non-empty value for `payment_order_id` but received {payment_order_id!r}")
        if not reversal_id:
            raise ValueError(f"Expected a non-empty value for `reversal_id` but received {reversal_id!r}")
        return self._get(
            f"/api/payment_orders/{payment_order_id}/reversals/{reversal_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Reversal,
        )

    def list(
        self,
        payment_order_id: str,
        *,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPage[Reversal]:
        """
        Get a list of all reversals of a payment order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not payment_order_id:
            raise ValueError(f"Expected a non-empty value for `payment_order_id` but received {payment_order_id!r}")
        return self._get_api_list(
            f"/api/payment_orders/{payment_order_id}/reversals",
            page=SyncPage[Reversal],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "per_page": per_page,
                    },
                    reversal_list_params.ReversalListParams,
                ),
            ),
            model=Reversal,
        )


class AsyncReversals(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncReversalsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReversalsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReversalsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncReversalsWithStreamingResponse(self)

    async def create(
        self,
        payment_order_id: str,
        *,
        reason: Literal[
            "duplicate",
            "incorrect_amount",
            "incorrect_receiving_account",
            "date_earlier_than_intended",
            "date_later_than_intended",
        ],
        ledger_transaction: reversal_create_params.LedgerTransaction | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> Reversal:
        """
        Create a reversal for a payment order.

        Args:
          payment_order_id: The id of the payment order being reversed.

          reason: The reason for the reversal. Must be one of `duplicate`, `incorrect_amount`,
              `incorrect_receiving_account`, `date_earlier_than_intended`,
              `date_later_than_intended`.

          ledger_transaction: Specifies a ledger transaction object that will be created with the reversal. If
              the ledger transaction cannot be created, then the reversal creation will fail.
              The resulting ledger transaction will mirror the status of the reversal.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not payment_order_id:
            raise ValueError(f"Expected a non-empty value for `payment_order_id` but received {payment_order_id!r}")
        return await self._post(
            f"/api/payment_orders/{payment_order_id}/reversals",
            body=await async_maybe_transform(
                {
                    "reason": reason,
                    "ledger_transaction": ledger_transaction,
                    "metadata": metadata,
                },
                reversal_create_params.ReversalCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=Reversal,
        )

    async def retrieve(
        self,
        reversal_id: str,
        *,
        payment_order_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Reversal:
        """
        Get details on a single reversal of a payment order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not payment_order_id:
            raise ValueError(f"Expected a non-empty value for `payment_order_id` but received {payment_order_id!r}")
        if not reversal_id:
            raise ValueError(f"Expected a non-empty value for `reversal_id` but received {reversal_id!r}")
        return await self._get(
            f"/api/payment_orders/{payment_order_id}/reversals/{reversal_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Reversal,
        )

    def list(
        self,
        payment_order_id: str,
        *,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Reversal, AsyncPage[Reversal]]:
        """
        Get a list of all reversals of a payment order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not payment_order_id:
            raise ValueError(f"Expected a non-empty value for `payment_order_id` but received {payment_order_id!r}")
        return self._get_api_list(
            f"/api/payment_orders/{payment_order_id}/reversals",
            page=AsyncPage[Reversal],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "per_page": per_page,
                    },
                    reversal_list_params.ReversalListParams,
                ),
            ),
            model=Reversal,
        )


class ReversalsWithRawResponse:
    def __init__(self, reversals: Reversals) -> None:
        self._reversals = reversals

        self.create = _legacy_response.to_raw_response_wrapper(
            reversals.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            reversals.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            reversals.list,
        )


class AsyncReversalsWithRawResponse:
    def __init__(self, reversals: AsyncReversals) -> None:
        self._reversals = reversals

        self.create = _legacy_response.async_to_raw_response_wrapper(
            reversals.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            reversals.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            reversals.list,
        )


class ReversalsWithStreamingResponse:
    def __init__(self, reversals: Reversals) -> None:
        self._reversals = reversals

        self.create = to_streamed_response_wrapper(
            reversals.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            reversals.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            reversals.list,
        )


class AsyncReversalsWithStreamingResponse:
    def __init__(self, reversals: AsyncReversals) -> None:
        self._reversals = reversals

        self.create = async_to_streamed_response_wrapper(
            reversals.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            reversals.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            reversals.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["BankSettings"]


class BankSettings(BaseModel):
    id: str

    backup_withholding_percentage: Optional[int] = None
    """The percentage of backup withholding to apply to the legal entity."""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    enable_backup_withholding: Optional[bool] = None
    """Whether backup withholding is enabled.

    See more here -
    https://www.irs.gov/businesses/small-businesses-self-employed/backup-withholding.
    """

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    privacy_opt_out: Optional[bool] = None
    """Cross River Bank specific setting to opt out of privacy policy."""

    regulation_o: Optional[bool] = None
    """
    It covers, among other types of insider loans, extensions of credit by a member
    bank to an executive officer, director, or principal shareholder of the member
    bank; a bank holding company of which the member bank is a subsidiary; and any
    other subsidiary of that bank holding company.
    """

    updated_at: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury._utils import parse_datetime
from modern_treasury.pagination import SyncPage, AsyncPage
from modern_treasury.types.ledger_transactions import LedgerTransactionVersion

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVersions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        version = client.ledger_transactions.versions.list()
        assert_matches_type(SyncPage[LedgerTransactionVersion], version, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        version = client.ledger_transactions.versions.list(
            after_cursor="after_cursor",
            created_at={"foo": parse_datetime("2019-12-27T18:11:19.117Z")},
            ledger_account_statement_id="ledger_account_statement_id",
            ledger_transaction_id="ledger_transaction_id",
            per_page=0,
            version={"foo": 0},
        )
        assert_matches_type(SyncPage[LedgerTransactionVersion], version, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.ledger_transactions.versions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = response.parse()
        assert_matches_type(SyncPage[LedgerTransactionVersion], version, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.ledger_transactions.versions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = response.parse()
            assert_matches_type(SyncPage[LedgerTransactionVersion], version, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncVersions:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        version = await async_client.ledger_transactions.versions.list()
        assert_matches_type(AsyncPage[LedgerTransactionVersion], version, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        version = await async_client.ledger_transactions.versions.list(
            after_cursor="after_cursor",
            created_at={"foo": parse_datetime("2019-12-27T18:11:19.117Z")},
            ledger_account_statement_id="ledger_account_statement_id",
            ledger_transaction_id="ledger_transaction_id",
            per_page=0,
            version={"foo": 0},
        )
        assert_matches_type(AsyncPage[LedgerTransactionVersion], version, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.ledger_transactions.versions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        version = response.parse()
        assert_matches_type(AsyncPage[LedgerTransactionVersion], version, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.ledger_transactions.versions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            version = await response.parse()
            assert_matches_type(AsyncPage[LedgerTransactionVersion], version, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from .._models import BaseModel
from .account_detail import AccountDetail
from .routing_detail import RoutingDetail

__all__ = ["VirtualAccount"]


class VirtualAccount(BaseModel):
    id: str

    account_details: List[AccountDetail]
    """An array of account detail objects."""

    counterparty_id: Optional[str] = None
    """The ID of a counterparty that the virtual account belongs to. Optional."""

    created_at: datetime

    credit_ledger_account_id: Optional[str] = None
    """The ID of a credit normal ledger account.

    When money enters the virtual account, this ledger account will be credited.
    Must be accompanied by a debit_ledger_account_id if present.
    """

    debit_ledger_account_id: Optional[str] = None
    """The ID of a debit normal ledger account.

    When money enters the virtual account, this ledger account will be debited. Must
    be accompanied by a credit_ledger_account_id if present.
    """

    description: Optional[str] = None
    """An optional free-form description for internal use."""

    discarded_at: Optional[datetime] = None

    internal_account_id: str
    """The ID of the internal account that the virtual account is in."""

    ledger_account_id: Optional[str] = None
    """
    If the virtual account links to a ledger account in Modern Treasury, the id of
    the ledger account will be populated here.
    """

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: str
    """The name of the virtual account."""

    object: str

    routing_details: List[RoutingDetail]
    """An array of routing detail objects.

    These will be the routing details of the internal account.
    """

    updated_at: datetime
